# Generated by Django 3.2.13 on 2022-07-25 10:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('messaging', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topic', models.CharField(help_text='Topic you are subscribing to.', max_length=128)),
                ('arn', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True, db_index=True)),
            ],
            options={
                'verbose_name': 'Topic',
                'verbose_name_plural': 'Topics',
            },
        ),
        migrations.AlterField(
            model_name='topiccallback',
            name='arn',
            field=models.CharField(blank=True, editable=False, max_length=255, null=True),
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.JSONField()),
                ('subject', models.CharField(blank=True, max_length=128, null=True)),
                ('object_id', models.PositiveIntegerField(editable=False)),
                ('message_id', models.UUIDField(null=True)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('content_type', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='sent_messages', to='contenttypes.contenttype')),
                ('topic', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sent_messages', to='messaging.topic')),
            ],
            options={
                'verbose_name': 'SentMessage',
                'verbose_name_plural': 'SentMessages',
            },
        ),
        migrations.AlterField(
            model_name='topiccallback',
            name='topic',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscriptions', to='messaging.topic'),
        ),
    ]



from rest_framework import serializers


class FieldMixin(object):
    """
       Turning to mixin / This class maybe extended to futureproofing stuffs/
       Can override the get_fields_names to get a specific field on our serializer

    """
    # TODO : TO be customized fully---> then make it more usable

    def get_fields_names(self, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        fields_names = self.context.get('fields', None)
        if fields_names:
            return fields_names
        return super(FieldMixin, self).get_fields_names(*args, **kwargs)


class APISerializer(serializers.BaseSerializer):

    def to_representation(self, data):
        success = True
        payload = data
        if isinstance(data, dict):
            if 'success' in data:
                success = data.pop('success')
            if 'payload' in data:
                payload = data.pop('payload')
            if 'reason' in data:
                reason = data.get('reason')
                self.context['reason'] = reason
        self.context['success'] = success

        return dict(self.context, **{
            'payload': payload})



# Generated by Django 3.2.13 on 2022-07-25 08:37

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TopicCallback',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topic', models.CharField(help_text='Topic you are subscribing to.', max_length=128)),
                ('protocol', models.CharField(choices=[('https', 'HTTPS'), ('http', 'HTTP'), ('email', 'Email'), ('lambda', 'AWS Lambda'), ('sqs', 'Amazon SQS')], max_length=32)),
                ('callback', models.URLField(help_text='Fully qualified callback URI that messages for the topic will     be pushed to', max_length=255)),
                ('enabled', models.BooleanField(default=True)),
                ('arn', models.CharField(blank=True, max_length=255, null=True)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True, db_index=True)),
            ],
            options={
                'verbose_name': 'Topic Callback',
                'verbose_name_plural': 'Topic Callbacks',
            },
        ),
    ]



import logging
import urllib.request


def check_and_register_sns(request, message):
    """
    Checks if an inbound message is an SNS confirm registration message, and then completes the registration
    :return:
    """
    if message.get('Type', 'None') not in ['SubscriptionConfirmation']:
        ''' Not a subscription request message '''
        return False

    confirmation_url = message.get('SubscribeURL')
    contents = urllib.request.urlopen(confirmation_url).read()
    return True


def check_target_for_common_update(message, model_ref):
    """ A common interface to receive targeted updates on request or update local record on non-targeted """
    from messaging.models import Message
    target = message.get('target', None)
    payload = message['payload']
    if isinstance(payload, dict):  # normalise to list of dicts if single item
        payload = [payload]

    if target is not None:
        if Message.objects.filter(message_id=target).exists() is False:
            ''' Targeted message not for us, so don't handle '''
            return ""

    for item in payload:
        if 'clients' in item:  # this relationship not required here. TODO perhaps use a serializer validation here
            drop = item.pop('clients')
        ''' remove data from item that does not exist in local serializer '''
        unhandled = list(set(item) - set(model_ref.__dict__.keys()))
        for _key in unhandled:
            drop = item.pop(_key)
        obj, created = model_ref.objects.update_or_create(id=item['id'], defaults=item)
        logging.warning(f"{obj}, {created}")



import setuptools

setuptools.setup(
    # name='4sure_message_wrapper',
    # version='0.0.1',
    # author='Andrew Simpson',
    # author_email='andrew.simpson@4-sure.net',
    # description='Message wrapper to add messaging functionality to our microservices',
    # long_description=long_description,
    # long_description_content_type="text/markdown",
    # url='https://bitbucket.org/101collective/4sure_message_wrapper',
    # project_urls = {
    #     "Bug Tracker": "https://bitbucket.org/101collective/4sure_message_wrapper/issues"
    # },
    # license='MIT',
    # packages=['messages'],
    # install_requires=['requests', 'boto3'],
)



import json
import os
import re
from typing import Union
import logging

import boto3
from botocore.exceptions import ClientError
from django.conf import settings

from . import MessageWrapper as MessageWrapperBase


class MessageWrapper(MessageWrapperBase):

    def __init__(self, **kwargs):
        region: str = kwargs.get('region', 'af-south-1')
        self.sns_resource = boto3.resource('sns', region_name=region)

    def create_topic(self, topic: str):
        """
        Create a topic with the supplied topic, environment aware
        """
        try:
            _topic = self.sns_resource.create_topic(Name=self._get_env_topic(topic))
        except ClientError as e:
            raise
        else:
            return _topic

    def subscribe(self, topic: str, protocol: str = 'http', endpoint: str = None) -> bool:
        """
        Get a topic ARN by topic
        """
        _topic = self.create_topic(topic=topic)
        try:
            arn = _topic.subscribe(
                Protocol=protocol,
                Endpoint=endpoint,
                # Attributes={'cliennt': 'sil', 'environment': 'DEV1'},
                ReturnSubscriptionArn=True
            )
        except ClientError:
            raise
        return arn

    def unsubscribe(self, topic: str, topic_link: str = None, *args, **kwargs) -> bool:
        pass

    def publish(self, topic: str, message: Union[str, dict], subject: str = None, payload: dict = None, *args, **kwargs):
        _topic = self.create_topic(topic)
        payload = dict() if payload is None else payload
        payload['payload'] = message
        self._inject_source_name(payload)
        params = dict(Message=json.dumps(payload))
        if subject is not None:
            params['Subject'] = subject
        res = _topic.publish(**params)
        return res

    def delete_topic(self, topic: str) -> bool:
        pass

    def _get_env_topic(self, topic):
        env = os.environ.get('RUNTIME_ENVIRON',
                             settings.__dict__.get('RUNTIME_ENVIRON', settings.__dict__.get('MAVEN_SECTION', 'DEV')))
        logging.warning(f'Using environment {env}')
        env = re.sub('[^A-Za-z_]', '', env)  # remove non-alpha chars
        env_topic = f"{env.lower()}_{topic}"
        return env_topic

    @staticmethod
    def _inject_source_name(params):
        source = settings.__dict__.get('MICROSERVICE_NAME', settings.__dict__.get('CLIENT', 'unknown'))
        params['source'] = source

